@@ -17,18 +17,23 @@
  */
 #include "config.h"
 
-#include <gnome.h>
-
-#include <stdlib.h>
-#include <stdio.h>
 #include <errno.h>
+#include <gnome.h>
+#include <sched.h>
+#include <stdio.h>
+#include <stdlib.h>
 #include <string.h>
 
+#include "app.h"
 #include "ctree.h"
 #include "cur-proj.h"
 #include "gtt.h"
+#include "log.h"
 #include "menucmd.h"
+#include "menus.h"
+#include "prefs.h"
 #include "props-proj.h"
+#include "timer.h"
 #include "toolbar.h"
 #include "util.h"
 
@@ -45,12 +50,12 @@
 
 
 GttProject *cur_proj = NULL;
-GtkWidget *glist, *window;
-#ifndef GNOME_USE_APP
-GtkBox *window_vbox;
-#endif
 
+ProjTreeWindow *global_ptw;
+GtkWidget *window;
+GtkWidget *glist;
 GtkWidget *status_bar;
+
 #ifdef GTK_USE_STATUSBAR
 static GtkStatusbar *status_project = NULL;
 static GtkStatusbar *status_day_time = NULL;
@@ -73,52 +78,54 @@ void update_status_bar(void)
 {
 	char day_total_str[25];
 	static char *old_day_time = NULL;
-        static char *old_project = NULL;
+	static char *old_project = NULL;
 	char *s;
 
 	if (!status_bar) return;
 	if (status_timer) {
-		if (timer_is_running())
-			gtk_widget_show(status_timer);
-		else
-			gtk_widget_hide(status_timer);
+	if (timer_is_running())
+		gtk_widget_show(status_timer);
+	else
+		gtk_widget_hide(status_timer);
 	}
-        if (!old_day_time) old_day_time = g_strdup("");
-        if (!old_project) old_project = g_strdup("");
+	if (!old_day_time) old_day_time = g_strdup("");
+	if (!old_project) old_project = g_strdup("");
 
 	print_hours_elapsed (day_total_str, 25, 
-		gtt_project_list_total_secs_day(), config_show_secs);
+	gtt_project_list_total_secs_day(), config_show_secs);
 
-        s = g_strdup(day_total_str);
-        if (0 != strcmp(s, old_day_time)) {
+	s = g_strdup(day_total_str);
+	if (0 != strcmp(s, old_day_time)) {
 #ifdef GTK_USE_STATUSBAR
-                gtk_statusbar_remove(status_day_time, 2, status_day_time_id);
-                status_day_time_id = gtk_statusbar_push(status_day_time, 2, s);
+		gtk_statusbar_remove(status_day_time, 2, status_day_time_id);
+		status_day_time_id = gtk_statusbar_push(status_day_time, 2, s);
 #else /* not GTK_USE_STATUSBAR */
-                gtk_label_set(status_day_time, s);
+		gtk_label_set(status_day_time, s);
 #endif /* not GTK_USE_STATUSBAR */
-                g_free(old_day_time);
-                old_day_time = s;
-        } else {
-                g_free(s);
-        }
-        if (cur_proj) {
-                s = g_strdup(gtt_project_get_title(cur_proj));
-        } else {
-                s = g_strdup(_("no project selected"));
-        }
-        if (0 != strcmp(s, old_project)) {
+		g_free(old_day_time);
+		old_day_time = s;
+	} else {
+		g_free(s);
+	}
+	if (cur_proj) {
+		s = g_strdup_printf ("%s - %s", 
+		gtt_project_get_title(cur_proj),
+		gtt_project_get_desc(cur_proj));
+	} else {
+		s = g_strdup(_("no project selected"));
+	}
+	if (0 != strcmp(s, old_project)) {
 #ifdef GTK_USE_STATUSBAR
-                gtk_statusbar_remove(status_project, 1, status_project_id);
-                status_project_id = gtk_statusbar_push(status_project, 1, s);
+		gtk_statusbar_remove(status_project, 1, status_project_id);
+		status_project_id = gtk_statusbar_push(status_project, 1, s);
 #else /* not GTK_USE_STATUSBAR */
-                gtk_label_set(status_project, s);
+		gtk_label_set(status_project, s);
 #endif /* not GTK_USE_STATUSBAR */
-                g_free(old_project);
-                old_project = s;
-        } else {
-                g_free(s);
-        }
+		g_free(old_project);
+		old_project = s;
+	} else {
+		g_free(s);
+	}
 }
 
 
@@ -126,50 +133,55 @@ void update_status_bar(void)
 void 
 cur_proj_set(GttProject *proj)
 {
+	GttProject *prev_proj = NULL;
 	pid_t pid;
-	char *cmd, *p;
-	GString *str;
+	char *cmd;
+	const char *str;
 
+	/* Due to the way the widget callbacks work, 
+	 * we may be called recursively ... */
 	if (cur_proj == proj) return;
 
 	log_proj(NULL);
-	stop_timer();
-	cur_proj = proj;
+	gtt_project_timer_stop (cur_proj);
 	if (proj) 
 	{
-		start_timer(); 
+		cur_proj = proj;
+		gtt_project_timer_start (proj); 
+		ctree_select (global_ptw, proj);
+	}
+	else
+	{
+		prev_proj = cur_proj;
+		cur_proj = NULL;
+		if (prev_proj) ctree_unselect (global_ptw, prev_proj);
 	}
 	log_proj(proj);
 	menu_set_states();
 	prop_dialog_set_project(proj);
 	update_status_bar();
 	cmd = (proj) ? config_command : config_command_null;
+
+	/* handle commands */
 	if (!cmd) return;
-	str = g_string_new (NULL);
-	for (p = cmd; *p; p++) {
-		if ((p[0] == '%') && (p[1] == 's')) {
-			if (cur_proj)
-			{
-				g_string_append (str, gtt_project_get_title(cur_proj));
-			}
-			p++;
-		} else {
-			g_string_append_c (str, *p);
-		}
-	}
+	str = printf_project (cmd, (proj) ? cur_proj : prev_proj);
 	pid = fork();
-	if (pid == 0) {
-		/* if we can't fork exec in first child */
-		if (fork () <= 0) {
-			execlp("sh", "sh", "-c", str->str, NULL);
-			g_warning("%s: %d: cur_proj_set: couldn't exec\n", __FILE__, __LINE__);
-			exit(1);
-		}
-		exit (0);
+	if (pid == 0) 
+	{
+		execlp("sh", "sh", "-c", str, NULL);
+		g_warning("%s: %d: cur_proj_set: couldn't exec\n", __FILE__, __LINE__);
+		exit(1);
 	}
 	if (pid < 0) {
 		g_warning("%s: %d: cur_proj_set: couldn't fork\n", __FILE__, __LINE__);
 	}
+
+	/* Note that the forked processes might be scheduled by the operating
+	 * system 'out of order', if we've made rapid successive calls to this
+	 * routine.  So we try to ensure in-order execution by trying to let
+	 * the child process at least start running.  And we can do this by
+	 * yielding our time-slice ... */
+	sched_yield();
 }
 
 
@@ -181,42 +193,40 @@ void app_new(int argc, char *argv[], const char *geometry_string)
 	gint x, y, w, h;
 
 	window = gnome_app_new("gtt", APP_NAME " " VERSION);
-        gtk_window_set_wmclass(GTK_WINDOW(window),
-                               "gtt", "GTimeTracker");
+	gtk_window_set_wmclass(GTK_WINDOW(window),
+			       "gtt", "GTimeTracker");
 	/* 320 x 220 seems to be a good size to default to */
 	gtk_window_set_default_size(GTK_WINDOW(window), 320, 220);
 	gtk_window_set_policy(GTK_WINDOW(window), TRUE, TRUE, FALSE);
 	menus_create(GNOME_APP(window));
 	widget = build_toolbar();
-        gtk_widget_show(widget);
-        gnome_app_set_toolbar(GNOME_APP(window), GTK_TOOLBAR(widget));
+	gtk_widget_show(widget);
+	gnome_app_set_toolbar(GNOME_APP(window), GTK_TOOLBAR(widget));
 	vbox = gtk_vbox_new(FALSE, 0);
 
 	status_bar = gtk_hbox_new(FALSE, 0);
 	gtk_widget_show(status_bar);
 	gtk_box_pack_end(GTK_BOX(vbox), status_bar, FALSE, FALSE, 2);
-        status_day_time = GTK_STATUSBAR(gtk_statusbar_new());
+	status_day_time = GTK_STATUSBAR(gtk_statusbar_new());
 #ifdef SB_USE_HACK
-        gtk_widget_show(GTK_WIDGET(status_day_time->frame));
-        gtk_widget_show(GTK_WIDGET(status_day_time->label));
+	gtk_widget_show(GTK_WIDGET(status_day_time->frame));
+	gtk_widget_show(GTK_WIDGET(status_day_time->label));
 #endif /* SB_USE_HACK */
-        gtk_widget_show(GTK_WIDGET(status_day_time));
-        status_day_time_id = gtk_statusbar_push(status_day_time,
-						2,
-						_("00:00"));
-        gtk_box_pack_start(GTK_BOX(status_bar), GTK_WIDGET(status_day_time),
-                           FALSE, FALSE, 1);
-        status_project = GTK_STATUSBAR(gtk_statusbar_new());
+	gtk_widget_show(GTK_WIDGET(status_day_time));
+	status_day_time_id = gtk_statusbar_push(status_day_time,
+						2, _("00:00"));
+	gtk_box_pack_start(GTK_BOX(status_bar), GTK_WIDGET(status_day_time),
+			   FALSE, FALSE, 1);
+	status_project = GTK_STATUSBAR(gtk_statusbar_new());
 #ifdef SB_USE_HACK
-        gtk_widget_show(GTK_WIDGET(status_project->frame));
-        gtk_widget_show(GTK_WIDGET(status_project->label));
+	gtk_widget_show(GTK_WIDGET(status_project->frame));
+	gtk_widget_show(GTK_WIDGET(status_project->label));
 #endif /* SB_USE_HACK */
-        gtk_widget_show(GTK_WIDGET(status_project));
-        status_project_id = gtk_statusbar_push(status_project,
-					       1,
-					       _("no project selected"));
-        gtk_box_pack_start(GTK_BOX(status_bar), GTK_WIDGET(status_project),
-                           TRUE, TRUE, 1);
+	gtk_widget_show(GTK_WIDGET(status_project));
+	status_project_id = gtk_statusbar_push(status_project,
+					       1, _("no project selected"));
+	gtk_box_pack_start(GTK_BOX(status_bar), GTK_WIDGET(status_project),
+			   TRUE, TRUE, 1);
 	status_timer = gnome_stock_pixmap_widget_at_size(GTK_WIDGET(status_bar),
 							 GNOME_STOCK_PIXMAP_TIMER,
 							 16, 16);
@@ -224,10 +234,10 @@ void app_new(int argc, char *argv[], const char *geometry_string)
 	gtk_box_pack_end(GTK_BOX(status_bar), GTK_WIDGET(status_timer),
 			 FALSE, FALSE, 1);
 
-        glist = create_ctree();
+	global_ptw = ctree_new();
+	glist = ctree_get_widget(global_ptw);
+
 	gtk_box_pack_end(GTK_BOX(vbox), glist->parent, TRUE, TRUE, 0);
-	gtk_widget_set_usize(glist, -1, 120);
-	gtk_widget_show_all(glist->parent);
 
 	gtk_widget_show(vbox);
 	gnome_app_set_contents(GNOME_APP(window), vbox);
@@ -247,7 +257,16 @@ void app_new(int argc, char *argv[], const char *geometry_string)
 	} else {
 		gnome_app_error(GNOME_APP(window),
 			_("Couldn't understand geometry (position and size)\n"
-				" specified on command line"));
+			  " specified on command line"));
+	}
+}
+
+void 
+app_show (void)
+{
+	if (!GTK_WIDGET_MAPPED(window)) 
+	{
+		gtk_widget_show(window);
 	}
 }
 
