@@ -84,11 +84,17 @@ typedef struct ProjTreeNode_s
 struct ProjTreeWindow_s 
 {
 	GtkCTree *ctree;
+
+	/* stuff that defines the column layout */
 	ColType cols[NCOLS];
 	char * col_titles[NCOLS];
+	char * col_tooltips[NCOLS];
+	GtkTooltips *col_tt_w[NCOLS];
 	GtkJustification col_justify[NCOLS];
+	gboolean col_width_set[NCOLS];
 	int ncols;
 
+	/* stuff we need to have handy while dragging */
 	GdkDragContext *drag_context;
 	GtkCTreeNode *source_ctree_node;
 	GtkCTreeNode *parent_ctree_node;
@@ -250,6 +256,8 @@ tree_collapse (GtkCTree *ctree, GtkCTreeNode *row)
 	}
 }
 
+/* ============================================================== */
+
 static void
 click_column(GtkCList *clist, gint col, gpointer data)
 {
@@ -317,31 +325,6 @@ click_column(GtkCList *clist, gint col, gpointer data)
 	ctree_setup(ptw);
 }
 
-#ifdef CLIST_HEADER_HACK
-static void
-clist_header_hack(GtkWidget *w)
-{
-	GtkStyle *style;
-	GdkGCValues vals;
-	int width;
-
-	if (clist_header_width_set) return;
-	clist_header_width_set = 1;
-	style = gtk_widget_get_style(w);
-	g_return_if_fail(style != NULL);
-	if (!style->fg_gc[0]) {
-		/* Fallback if the GC still isn't available */
-		width = 50;
-	} else {
-		gdk_gc_get_values(style->fg_gc[0], &vals);
-		width = gdk_string_width(vals.font, "00:00:00");
-	}
-	gtk_clist_set_column_width(GTK_CLIST(w), TOTAL_COL, width);
-	gtk_clist_set_column_width(GTK_CLIST(w), TIME_COL, width);
-	gtk_clist_set_column_width(GTK_CLIST(w), TITLE_COL, 120);
-}
-#endif /* CLIST_HEADER_HACK */
-
 /* ============================================================== */
 /* Attempt to change pixmaps to indicate whether dragged project
  * will be a peer, or a child, of the project its dragged onto.
@@ -525,75 +508,137 @@ ctree_init_cols (ProjTreeWindow *ptw)
 		{
 			case TIME_EVER_COL:
 				ptw->col_justify[i] = GTK_JUSTIFY_CENTER;
-				ptw->col_titles[i] =  _("Total");
+				ptw->col_titles[i] = _("Total");
+				ptw->col_tooltips[i] = 
+					_("Total time spent on this project.");
+				ptw->col_width_set[i] = FALSE;
 				break;
 			case TIME_CURRENT_COL:
 				ptw->col_justify[i] = GTK_JUSTIFY_CENTER;
-				ptw->col_titles[i] =  _("This Memo");
+				ptw->col_titles[i] = _("This Memo");
+				ptw->col_tooltips[i] = 
+					_("Time spent under the current "
+					"task memo.");
+				ptw->col_width_set[i] = FALSE;
 				break;
 			case TIME_TODAY_COL:
 				ptw->col_justify[i] = GTK_JUSTIFY_CENTER;
-				ptw->col_titles[i] =  _("Today");
+				ptw->col_titles[i] = _("Today");
+				ptw->col_tooltips[i] =  
+					_("Time spent on this project today.");
+				ptw->col_width_set[i] = FALSE;
 				break;
 			case TIME_WEEK_COL:
 				ptw->col_justify[i] = GTK_JUSTIFY_CENTER;
-				ptw->col_titles[i] =  _("Week");
+				ptw->col_titles[i] = _("Week");
+				ptw->col_tooltips[i] = 
+					_("Time spent on this project "
+					"this week.");
+				ptw->col_width_set[i] = FALSE;
 				break;
 			case TIME_MONTH_COL:
 				ptw->col_justify[i] = GTK_JUSTIFY_CENTER;
 				ptw->col_titles[i] =  _("Month");
+				ptw->col_tooltips[i] =  
+					_("Time spent on this project "
+					"this month.");
+				ptw->col_width_set[i] = FALSE;
 				break;
 			case TIME_YEAR_COL:
 				ptw->col_justify[i] = GTK_JUSTIFY_CENTER;
 				ptw->col_titles[i] =  _("Year");
+				ptw->col_tooltips[i] = 
+					_("Time spent on this project "
+					"this year.");
+				ptw->col_width_set[i] = FALSE;
 				break;
 			case TITLE_COL:
 				ptw->col_justify[i] = GTK_JUSTIFY_LEFT;
-				ptw->col_titles[i] =  _("Project Title");
+				ptw->col_titles[i] =  _("Title");
+				ptw->col_tooltips[i] =  _("Project Title");
+				ptw->col_width_set[i] = FALSE;
 				break;
 			case DESC_COL:
 				ptw->col_justify[i] = GTK_JUSTIFY_LEFT;
 				ptw->col_titles[i] =  _("Description");
+				ptw->col_tooltips[i] =  _("Description");
+				ptw->col_width_set[i] = FALSE;
 				break;
 			case TASK_COL:
 				ptw->col_justify[i] = GTK_JUSTIFY_LEFT;
 				ptw->col_titles[i] =  _("Task");
+				ptw->col_tooltips[i] = 
+					_("The memo for the current task");
+				ptw->col_width_set[i] = FALSE;
 				break;
 			case START_COL:
-				ptw->col_justify[i] = GTK_JUSTIFY_LEFT;
-				ptw->col_titles[i] =  _("Estimated Start Date");
+				ptw->col_justify[i] = GTK_JUSTIFY_CENTER;
+				ptw->col_titles[i] =  _("Start");
+				ptw->col_tooltips[i] =  
+					_("Estimated Project Start Date");
+				ptw->col_width_set[i] = FALSE;
 				break;
 			case END_COL:
-				ptw->col_justify[i] = GTK_JUSTIFY_LEFT;
-				ptw->col_titles[i] =  _("Estimated End Date");
+				ptw->col_justify[i] = GTK_JUSTIFY_CENTER;
+				ptw->col_titles[i] =  _("End");
+				ptw->col_tooltips[i] =  
+					_("Estimated Project Completion Date");
+				ptw->col_width_set[i] = FALSE;
 				break;
 			case DUE_COL:
-				ptw->col_justify[i] = GTK_JUSTIFY_LEFT;
-				ptw->col_titles[i] =  _("Date Due");
+				ptw->col_justify[i] = GTK_JUSTIFY_CENTER;
+				ptw->col_titles[i] =  _("Due");
+				ptw->col_tooltips[i] =  
+					_("Due Date: the date by which "
+					"the project absolutely must be "
+					"completed by.");
+				ptw->col_width_set[i] = FALSE;
 				break;
 			case SIZING_COL:
-				ptw->col_justify[i] = GTK_JUSTIFY_LEFT;
-				ptw->col_titles[i] =  _("Sizing");
+				ptw->col_justify[i] = GTK_JUSTIFY_RIGHT;
+				ptw->col_titles[i] = _("Size");
+				ptw->col_tooltips[i] = 
+					_("Sizing: How much work it will "
+					"take to finish this project.");
+				ptw->col_width_set[i] = FALSE;
 				break;
 			case PERCENT_COL:
-				ptw->col_justify[i] = GTK_JUSTIFY_LEFT;
-				ptw->col_titles[i] =  _("% Complete");
+				ptw->col_justify[i] = GTK_JUSTIFY_RIGHT;
+				ptw->col_titles[i] =  _("Done");
+				ptw->col_tooltips[i] = 
+					_("Percent Complete: How much "
+					"of this project has been "
+					"finished till now.");
+				ptw->col_width_set[i] = FALSE;
 				break;
 			case URGENCY_COL:
 				ptw->col_justify[i] = GTK_JUSTIFY_LEFT;
-				ptw->col_titles[i] =  _("Urgency");
+				ptw->col_titles[i] = _("U");
+				ptw->col_tooltips[i] = 
+					_("Urgency: Does this project need "
+					"to be done soon, done quickly?");
+				ptw->col_width_set[i] = FALSE;
 				break;
 			case IMPORTANCE_COL:
 				ptw->col_justify[i] = GTK_JUSTIFY_LEFT;
-				ptw->col_titles[i] =  _("Importance");
+				ptw->col_titles[i] = _("Imp");
+				ptw->col_tooltips[i] = 
+					_("Importance: How important "
+					"is this project?");
+				ptw->col_width_set[i] = FALSE;
 				break;
 			case STATUS_COL:
 				ptw->col_justify[i] = GTK_JUSTIFY_LEFT;
-				ptw->col_titles[i] =  _("Status");
+				ptw->col_titles[i] = _("Status");
+				ptw->col_tooltips[i] = 
+					_("The current status of this project.");
+				ptw->col_width_set[i] = FALSE;
 				break;
 			case NULL_COL:
 				ptw->col_justify[i] = GTK_JUSTIFY_LEFT;
 				ptw->col_titles[i] =  "";
+				ptw->col_tooltips[i] =  "";
+				ptw->col_width_set[i] = FALSE;
 				break;
 		}
 	}
@@ -601,6 +646,69 @@ ctree_init_cols (ProjTreeWindow *ptw)
 
 /* ============================================================== */
 
+static int
+string_width(GtkWidget *w, const char * str)
+{
+	GtkStyle *style;
+	GdkFont *font;
+	GdkGC *gc;
+	GdkGCValues vals;
+	int width;
+
+	style = gtk_widget_get_style(w);
+
+	/* Fallback if the GC still isn't available */
+	if (NULL == style) 
+	{
+		style = gtk_widget_get_default_style ();
+		if (NULL == style) return 49;
+	}
+	font = style->font;
+
+	/* glurg. Try really hard to find a font */
+	if (NULL == font)
+	{
+		gc = style->text_gc[0];
+		if (NULL == gc) gc = style->fg_gc[0];
+		if (NULL == gc) gc = style->bg_gc[0];
+		if (NULL == gc) gc = style->base_gc[0];
+		if (NULL == gc)
+		{
+			style = gtk_widget_get_default_style ();
+			if (NULL == style) return 50;
+	
+			gc = style->text_gc[0];
+			if (NULL == gc) gc = style->fg_gc[0];
+			if (NULL == gc) gc = style->bg_gc[0];
+			if (NULL == gc) gc = style->base_gc[0];
+			if (NULL == gc) return 51;
+		}
+	
+		gdk_gc_get_values(gc, &vals);
+		font = vals.font;
+	}
+
+	/* ha. Finally. Get the width. */
+	width = gdk_string_width(font, str);
+
+	return width;
+}
+
+/* Initialize to a reasonable column width, but only 
+ * if it hasn't been set already.  */
+
+static void
+default_col_width (ProjTreeWindow *ptw, int col, const char * str)
+{
+	int width;
+	if (TRUE == ptw->col_width_set[col]) return;
+
+	width = string_width (GTK_WIDGET(ptw->ctree), str);
+	ctree_set_col_width (ptw, col, width);
+}
+
+/* Note: we don't need i18n/l7n translations for most of the strings;
+ * they are used only for column widths ... */
 void 
 ctree_update_column_visibility (ProjTreeWindow *ptw)
 {
@@ -612,74 +720,92 @@ ctree_update_column_visibility (ProjTreeWindow *ptw)
 		switch (ptw->cols[i])
 		{
 		case TITLE_COL:
+			default_col_width (ptw, i, "-00:00:00");
 			break;
 		case TIME_EVER_COL:
-			gtk_clist_set_column_visibility (GTK_CLIST(ptw->ctree), i, 
-				config_show_title_ever);
+			default_col_width (ptw, i, "-00:00:00");
+			gtk_clist_set_column_visibility (GTK_CLIST(ptw->ctree), 
+				i, config_show_title_ever);
 			break;
 		case TIME_CURRENT_COL:
-			gtk_clist_set_column_visibility (GTK_CLIST(ptw->ctree), i, 
-				config_show_title_current);
+			default_col_width (ptw, i, "-00:00:00");
+			gtk_clist_set_column_visibility (GTK_CLIST(ptw->ctree),
+				i, config_show_title_current);
 			break;
 		case TIME_TODAY_COL:
-			gtk_clist_set_column_visibility (GTK_CLIST(ptw->ctree), i, 
-				config_show_title_day);
+			default_col_width (ptw, i, "-00:00:00");
+			gtk_clist_set_column_visibility (GTK_CLIST(ptw->ctree),
+				 i, config_show_title_day);
 			break;
 		case TIME_WEEK_COL:
-			gtk_clist_set_column_visibility (GTK_CLIST(ptw->ctree), i, 
-				config_show_title_week);
+			default_col_width (ptw, i, "-00:00:00");
+			gtk_clist_set_column_visibility (GTK_CLIST(ptw->ctree),
+				i, config_show_title_week);
 			break;
 		case TIME_MONTH_COL:
-			gtk_clist_set_column_visibility (GTK_CLIST(ptw->ctree), i, 
-				config_show_title_month);
+			default_col_width (ptw, i, "-00:00:00");
+			gtk_clist_set_column_visibility (GTK_CLIST(ptw->ctree), 
+				i, config_show_title_month);
 			break;
 		case TIME_YEAR_COL:
-			gtk_clist_set_column_visibility (GTK_CLIST(ptw->ctree), i, 
-				config_show_title_year);
+			default_col_width (ptw, i, "-00:00:00");
+			gtk_clist_set_column_visibility (GTK_CLIST(ptw->ctree),
+				i, config_show_title_year);
 			break;
 		case DESC_COL:
-			gtk_clist_set_column_visibility (GTK_CLIST(ptw->ctree), i, 
-				config_show_title_desc);
+			default_col_width (ptw, i, "Not too long");
+			gtk_clist_set_column_visibility (GTK_CLIST(ptw->ctree), 
+				i, config_show_title_desc);
 			break;
 		case TASK_COL:
-			gtk_clist_set_column_visibility (GTK_CLIST(ptw->ctree), i, 
-				config_show_title_task);
+			default_col_width (ptw, i, "Some longer string");
+			gtk_clist_set_column_visibility (GTK_CLIST(ptw->ctree),
+				i, config_show_title_task);
 			break;
 		case START_COL:
-			gtk_clist_set_column_visibility (GTK_CLIST(ptw->ctree), i, 
-				config_show_title_estimated_start);
+			default_col_width (ptw, i, "-00:00:00");
+			gtk_clist_set_column_visibility (GTK_CLIST(ptw->ctree),
+				i, config_show_title_estimated_start);
 			break;
 		case END_COL:
-			gtk_clist_set_column_visibility (GTK_CLIST(ptw->ctree), i, 
-				config_show_title_estimated_end);
+			default_col_width (ptw, i, "-00:00:00");
+			gtk_clist_set_column_visibility (GTK_CLIST(ptw->ctree),
+				i, config_show_title_estimated_end);
 			break;
 		case DUE_COL:
-			gtk_clist_set_column_visibility (GTK_CLIST(ptw->ctree), i, 
-				config_show_title_due_date);
+			default_col_width (ptw, i, "-00:00:00");
+			gtk_clist_set_column_visibility (GTK_CLIST(ptw->ctree),
+				i, config_show_title_due_date);
 			break;
 		case SIZING_COL:
-			gtk_clist_set_column_visibility (GTK_CLIST(ptw->ctree), i, 
-				config_show_title_sizing);
+			default_col_width (ptw, i, "XXX.XX");
+			gtk_clist_set_column_visibility (GTK_CLIST(ptw->ctree),
+				i, config_show_title_sizing);
 			break;
 		case PERCENT_COL:
-			gtk_clist_set_column_visibility (GTK_CLIST(ptw->ctree), i, 
-				config_show_title_percent_complete);
+			default_col_width (ptw, i, "100%");
+			gtk_clist_set_column_visibility (GTK_CLIST(ptw->ctree),
+				i, config_show_title_percent_complete);
 			break;
 		case URGENCY_COL:
-			gtk_clist_set_column_visibility (GTK_CLIST(ptw->ctree), i, 
-				config_show_title_urgency);
+			default_col_width (ptw, i, "X");
+			gtk_clist_set_column_visibility (GTK_CLIST(ptw->ctree),
+				i, config_show_title_urgency);
 			break;
 		case IMPORTANCE_COL:
-			gtk_clist_set_column_visibility (GTK_CLIST(ptw->ctree), i, 
-				config_show_title_importance);
+			default_col_width (ptw, i, "XXX");
+			gtk_clist_set_column_visibility (GTK_CLIST(ptw->ctree),
+				i, config_show_title_importance);
 			break;
 		case STATUS_COL:
-			gtk_clist_set_column_visibility (GTK_CLIST(ptw->ctree), i, 
-				config_show_title_status);
+			default_col_width (ptw, i, "abcedfg");
+			gtk_clist_set_column_visibility (GTK_CLIST(ptw->ctree),
+				i, config_show_title_status);
 			break;
 		case NULL_COL:
 			break;
 		}
+
 	}
 }
 
@@ -694,8 +820,13 @@ ctree_update_column_visibility (ProjTreeWindow *ptw)
 		} else {						\
 			secs = gtt_project_total_secs_##SLOT(prj); 	\
 		}							\
-		print_hours_elapsed (ptn->SLOT##_timestr, 24, 		\
-			secs, config_show_secs);			\
+		if (0 < secs) { 					\
+			print_hours_elapsed (ptn->SLOT##_timestr, 24,	\
+				secs, config_show_secs);		\
+		} else {						\
+			ptn->SLOT##_timestr[0] = '-';			\
+			ptn->SLOT##_timestr[1] = 0x0;			\
+		}							\
 	}								\
 }
 
@@ -759,8 +890,12 @@ ctree_col_values (ProjTreeNode *ptn, gboolean expand)
 			if (config_show_title_estimated_start) {
 				time_t secs;
 				secs = gtt_project_get_estimated_start(prj);
-				print_hours_elapsed (ptn->start_timestr, 24, 
-					secs, config_show_secs);
+				if (0 < secs) {
+					print_date (ptn->start_timestr, 24, secs);
+				} else {
+					ptn->start_timestr[0] = '-';
+					ptn->start_timestr[1] = 0x0;
+				}
 			}
 			break;
 		case END_COL:
@@ -768,8 +903,12 @@ ctree_col_values (ProjTreeNode *ptn, gboolean expand)
 			if (config_show_title_estimated_end) {
 				time_t secs;
 				secs = gtt_project_get_estimated_end(prj);
-				print_hours_elapsed (ptn->end_timestr, 24, 
-					secs, config_show_secs);
+				if (0 < secs) {
+					print_date (ptn->end_timestr, 24, secs);
+				} else {
+					ptn->end_timestr[0] = '-';
+					ptn->end_timestr[1] = 0x0;
+				}
 			}
 			break;
 		case DUE_COL:
@@ -777,8 +916,12 @@ ctree_col_values (ProjTreeNode *ptn, gboolean expand)
 			if (config_show_title_due_date) {
 				time_t secs;
 				secs = gtt_project_get_due_date(prj);
-				print_hours_elapsed (ptn->due_timestr, 24, 
-					secs, config_show_secs);
+				if (0 < secs) {
+					print_date (ptn->due_timestr, 24, secs);
+				} else {
+					ptn->due_timestr[0] = '-';
+					ptn->due_timestr[1] = 0x0;
+				}
 			}
 			break;
 		case SIZING_COL:
@@ -786,8 +929,14 @@ ctree_col_values (ProjTreeNode *ptn, gboolean expand)
 			if (config_show_title_sizing) {
 				int sz;
 				sz = gtt_project_get_sizing(prj);
-				snprintf (ptn->sizing_str, 24, 
-					"%.2f", (((double)sz)/3600.0));
+				if (0 < sz)
+				{
+					snprintf (ptn->sizing_str, 24, 
+						"%.2f", (((double)sz)/3600.0));
+				} else {
+					ptn->sizing_str[0] = '-';
+					ptn->sizing_str[1] = 0x0;
+				}
 			}
 			break;
 		case PERCENT_COL:
@@ -843,6 +992,67 @@ ctree_col_values (ProjTreeNode *ptn, gboolean expand)
 	}
 }
 
+/* ============================================================== */
+/* redraw utility, recursively walks visibile project tree */
+
+static void
+refresh_list (ProjTreeWindow *ptw, GList *prjlist)
+{
+	GList *node;
+	GtkCTree *tree_w = ptw->ctree;
+
+	/* now, draw each project */
+	for (node = prjlist; node; node = node->next) 
+	{
+		int i;
+		gboolean expand;
+		ProjTreeNode *ptn;
+		GttProject *prj = node->data;
+
+		ptn = gtt_project_get_private_data (prj);
+		expand = GTK_CTREE_ROW(ptn->ctnode)->expanded;
+		ctree_col_values (ptn, expand);
+
+		for (i=0; i<ptw->ncols; i++)
+		{
+			gtk_ctree_node_set_text(tree_w, ptn->ctnode, 
+				i, ptn->col_values[i]);
+		}
+
+		/* should we show sub-projects? recurse */
+		if (expand)
+		{
+			prjlist = gtt_project_get_children (prj);
+			refresh_list (ptw, prjlist);
+		}
+	}
+}
+
+/* ============================================================== */
+/* redraw all */
+
+void
+ctree_refresh (ProjTreeWindow *ptw)
+{
+	GtkCTree *tree_w;
+	GList *prjlist;
+
+	if (!ptw) return;
+	tree_w = ptw->ctree;
+
+	/* freeze, in prep for a massive update */
+	gtk_clist_freeze(GTK_CLIST(tree_w));
+
+	/* make sure the right set of columns are visibile */
+	ctree_update_column_visibility (ptw);
+
+	/* now, draw each project */
+	prjlist = gtt_get_project_list();
+	refresh_list (ptw, prjlist);
+
+	gtk_clist_thaw(GTK_CLIST(tree_w));
+}
+
 /* ============================================================== */
 /* redraw handler */
 
@@ -903,6 +1113,17 @@ ctree_new(void)
 	gtk_clist_column_titles_active(GTK_CLIST(w));
 	gtk_clist_set_selection_mode(GTK_CLIST(w), GTK_SELECTION_SINGLE);
 
+	/* some columns are quite narrow, so put tooltips over them. */
+	for (i=0; i<ptw->ncols; i++)
+	{
+		GtkTooltips *tt;
+		tt = gtk_tooltips_new();
+		gtk_tooltips_enable (tt);
+		gtk_tooltips_set_tip (tt, GTK_CLIST(w)->column[i].button,
+			ptw->col_tooltips[i], NULL);
+		ptw->col_tt_w[i] = tt;
+	}
+
 	gtk_widget_set_usize(w, -1, 120);
 	ctree_update_column_visibility (ptw);
 	gtk_ctree_set_show_stub(GTK_CTREE(w), FALSE);
@@ -972,10 +1193,12 @@ ctree_new(void)
 void
 ctree_setup (ProjTreeWindow *ptw)
 {
-	GtkCTree *tree_w = ptw->ctree;
+	GtkCTree *tree_w;
 	GList *node, *prjlist;
 	GttProject *running_project = cur_proj;
 
+	if (!ptw) return;
+	tree_w = ptw->ctree;
 	cur_proj_set (NULL);
 
 	/* first, add all projects to the ctree */
@@ -1291,7 +1514,7 @@ ctree_set_col_width (ProjTreeWindow *ptw, int col, int width)
 {
 	if (!ptw) return;
 	gtk_clist_set_column_width(GTK_CLIST(ptw->ctree), col, width);
-	// ptw->clist_header_width_set = 1;
+	ptw->col_width_set[col] = TRUE;
 }
 
 int
