@@ -11,7 +11,7 @@ fi
 
 (autoconf --version) < /dev/null > /dev/null 2>&1 || {
   echo
-  echo "**Error**: You must have \`autoconf' installed to compile Gnome."
+  echo "**Error**: You must have \`autoconf' installed to compile $PKG_NAME."
   echo "Download the appropriate package for your distribution,"
   echo "or get the source tarball at ftp://ftp.gnu.org/pub/gnu/"
   DIE=1
@@ -20,7 +20,7 @@ fi
 (grep "^AM_PROG_XML_I18N_TOOLS" $srcdir/configure.in >/dev/null) && {
   (xml-i18n-toolize --version) < /dev/null > /dev/null 2>&1 || {
     echo 
-    echo "**Error**: You must have \`xml-i18n-toolize' installed to compile Gnome."
+    echo "**Error**: You must have \`xml-i18n-toolize' installed to compile $PKG_NAME."
     echo "Get ftp://ftp.gnome.org/pub/GNOME/stable/sources/xml-i18n-tools/xml-i18n-tools-0.6.tar.gz"
     echo "(or a newer version if it is available)"
     DIE=1
@@ -30,7 +30,7 @@ fi
 (grep "^AM_PROG_LIBTOOL" $srcdir/configure.in >/dev/null) && {
   (libtool --version) < /dev/null > /dev/null 2>&1 || {
     echo
-    echo "**Error**: You must have \`libtool' installed to compile Gnome."
+    echo "**Error**: You must have \`libtool' installed to compile $PKG_NAME."
     echo "Get ftp://ftp.gnu.org/pub/gnu/libtool-1.2d.tar.gz"
     echo "(or a newer version if it is available)"
     DIE=1
@@ -41,8 +41,8 @@ fi
 #  grep "sed.*POTFILES" $srcdir/configure.in >/dev/null || \
 #  (gettext --version) < /dev/null > /dev/null 2>&1 || {
 #    echo
-#    echo "**Error**: You must have \`gettext' installed to compile Gnome."
-#    echo "Get ftp://alpha.gnu.org/gnu/gettext-0.10.35.tar.gz"
+#    echo "**Error**: You must have \`gettext' installed to compile $PKG_NAME."
+#    echo "Get ftp://ftp.gnu.org/pub/gnu/gettext/gettext-0.10.39.tar.gz"
 #    echo "(or a newer version if it is available)"
 #    DIE=1
 #  }
@@ -52,8 +52,8 @@ fi
 #  grep "sed.*POTFILES" $srcdir/configure.in >/dev/null || \
 #  (gettext --version) < /dev/null > /dev/null 2>&1 || {
 #    echo
-#    echo "**Error**: You must have \`gettext' installed to compile Gnome."
-#    echo "Get ftp://alpha.gnu.org/gnu/gettext-0.10.35.tar.gz"
+#    echo "**Error**: You must have \`gettext' installed to compile $PKG_NAME."
+#    echo "Get ftp://ftp.gnu.org/pub/gnu/gettext/gettext-0.10.39.tar.gz"
 #    echo "(or a newer version if it is available)"
 #    DIE=1
 #  }
@@ -61,7 +61,7 @@ fi
 
 (automake --version) < /dev/null > /dev/null 2>&1 || {
   echo
-  echo "**Error**: You must have \`automake' installed to compile Gnome."
+  echo "**Error**: You must have \`automake' installed to compile $PKG_NAME."
   echo "Get ftp://ftp.gnu.org/pub/gnu/automake-1.3.tar.gz"
   echo "(or a newer version if it is available)"
   DIE=1
